@@ -1,16 +1,24 @@
+#
+# Copyright (C) 2020 IHS Markit.
+# All Rights Reserved
+#
 import logging
 
 from dli.client.components.urls import package_urls
 from dli.client.components import SirenComponent
+from dli.client.exceptions import CatalogueEntityNotFoundException
 from dli.client.utils import ensure_count_is_valid, filter_out_unknown_keys, to_camel_cased_dict
+from dli.models import AttributesDict
+from dli.models.package_model import PackageModel
 from dli.siren import siren_to_dict, siren_to_entity
-from dli.models.dataset import DatasetModel
+from dli.models.dataset_model import DatasetModel
 
 logger = logging.getLogger(__name__)
 
-
 class Package(SirenComponent):
 
+    __USE_V2_ROUTE = False
+
     _KNOWN_FIELDS = {"name",
                      "description",
                      "keywords",
@@ -136,7 +144,7 @@ class Package(SirenComponent):
                             access manager_id will have to grant or deny access to the underlying data.
                             If access is flagged as `Unrestricted`, user will be able to gain
                             access instantaneously after submitting the access request form.
-        :param str internal_data: Accepted values are: `Yes`, `No` or `Both`. Package is marked as `Yes` if underlying data is created internally at IHS Markit, or No if externally, e.g. S&P, Russell, etc.
+        :param str internal_data: DEPRECATED - Note this is not used on V2 Accepted values are: `Yes`, `No` or `Both`. Package is marked as `Yes` if underlying data is created internally at IHS Markit, or No if externally, e.g. S&P, Russell, etc.
         :param str data_sensitivity: Accepted values are: `Private`, `Public` or `Top Secret`. Sensitivity level of the data contained within the package
         :param str terms_and_conditions: Terms and conditions for the package.
             Default T&C for IHSM organisations only can be obtained through
@@ -192,37 +200,83 @@ class Package(SirenComponent):
 
         self._validate_terms_and_conditions(terms_and_conditions)
 
-        payload = {
-            "name": name,
-            "description": description,
-            "keywords": keywords,
-            "topic": topic,
-            "access": access,
-            "internalData": internal_data,
-            "dataSensitivity": data_sensitivity,
-            "contractIds": contract_ids,
-            "termsAndConditions": terms_and_conditions,
-            "derivedDataNotes": derived_data_notes,
-            "derivedDataRights": derived_data_rights,
-            "distributionNotes": distribution_notes,
-            "distributionRights": distribution_rights,
-            "internalUsageNotes": internal_usage_notes,
-            "internalUsageRights": internal_usage_rights,
-            "documentation": documentation,
-            "publisher": publisher,
-            "techDataOpsId": tech_data_ops_id,
-            "accessManagerId": access_manager_id,
-            "managerId": manager_id,
-            "intendedPurpose": intended_purpose,
-            "isInternalWithinOrganisation": is_internal_within_organisation,
-        }
+        if Package.__USE_V2_ROUTE:
+            payload = {
+                "name": name,
+                "description": description,
+                "keywords": keywords,
+                "topic": topic,
+                "access": access,
+                "data_sensitivity": data_sensitivity,
+                "contract_ids": contract_ids,
+                "terms_and_conditions": terms_and_conditions,
+                "derived_data_notes": derived_data_notes,
+                "derived_data_rights": derived_data_rights,
+                "distribution_notes": distribution_notes,
+                "distribution_rights": distribution_rights,
+                "internal_usage_notes": internal_usage_notes,
+                "internal_usage_rights": internal_usage_rights,
+                "documentation": documentation,
+                "publisher": publisher,
+                "tech_data_ops_id": tech_data_ops_id,
+                "access_manager_id": access_manager_id,
+                "manager_id": manager_id,
+                "intended_purpose": intended_purpose,
+                "is_internal_within_organisation": is_internal_within_organisation,
+            }
 
-        payload = {k: v for k, v in payload.items() if v is not None}
-        return siren_to_entity(
-            self.session.post(
+            payload = {
+                "data": {
+                    "attributes": {
+                        k: v for k, v in payload.items() if v is not None
+                    }
+                }
+            }
+
+            a = self.session.post(
+                package_urls.v2_package_index, json=payload
+            ).json()
+            return self._Package(a)
+        else:
+            payload = {
+                "name": name,
+                "description": description,
+                "keywords": keywords,
+                "topic": topic,
+                "access": access,
+                "internalData": internal_data,
+                "dataSensitivity": data_sensitivity,
+                "contractIds": contract_ids,
+                "termsAndConditions": terms_and_conditions,
+                "derivedDataNotes": derived_data_notes,
+                "derivedDataRights": derived_data_rights,
+                "distributionNotes": distribution_notes,
+                "distributionRights": distribution_rights,
+                "internalUsageNotes": internal_usage_notes,
+                "internalUsageRights": internal_usage_rights,
+                "documentation": documentation,
+                "publisher": publisher,
+                "techDataOpsId": tech_data_ops_id,
+                "accessManagerId": access_manager_id,
+                "managerId": manager_id,
+                "intendedPurpose": intended_purpose,
+                "isInternalWithinOrganisation": is_internal_within_organisation,
+            }
+
+            payload = {k: v for k, v in payload.items() if v is not None}
+
+            if payload["internalData"] == None:
+                payload["internalData"] = 'No'
+
+            result = self.session.post(
                 package_urls.package_index, json=payload
             ).to_siren()
-        )
+            result.id = result.packageId
+
+            return siren_to_entity(
+                result
+            )
+
 
     def edit_package(
         self,
@@ -288,13 +342,14 @@ class Package(SirenComponent):
                     description="Updated my package description",
                 )
         """
-        fields = filter_out_unknown_keys(to_camel_cased_dict(kwargs), Package._KNOWN_FIELDS)
+        fields = filter_out_unknown_keys(to_camel_cased_dict(kwargs),
+                                         Package._KNOWN_FIELDS)
         self._validate_fields(fields)
 
+        # v2 package comes snake_case, so we must do some tom-jiggery
+        # back to camelCase from the model
         package = self._get_package(package_id=package_id)
-        # we can't just post back the siren object for some reason
-        # as it can't be deserialised
-        package_as_dict = siren_to_dict(package)
+        package_as_dict = to_camel_cased_dict(package.__dict__)
 
         # clean the package dict with fields that aren't known to us
         for key in list(package_as_dict.keys()):
@@ -302,6 +357,8 @@ class Package(SirenComponent):
                 del package_as_dict[key]
 
         package_as_dict.update(fields)
+        #v2 also misses 'internalData'
+        package_as_dict.update({'internalData': 'No'})
 
         result = self.session.put(
             package_urls.package_edit.format(id=package_id), json=package_as_dict
@@ -367,11 +424,25 @@ class Package(SirenComponent):
     # Private functions
     #
     def _get_package(self, **kwargs):
-        response = self.session.get(
-            package_urls.package_index, params=kwargs
-        )
 
-        return response.to_siren()
+        if 'package_id' in kwargs:
+            vals = self.packages(
+                search_term=[f"id={kwargs['package_id']}"]).values()
+            vals = list(vals)
+        elif 'name' in kwargs:
+            vals = self.packages(
+                search_term=[f"name={kwargs['name']}"]).get(kwargs["name"])
+            vals = [vals]
+
+        if len(vals) == 1:
+            if hasattr(vals[0], "id"):
+                vals[0].package_id = vals[0].id
+
+            return vals[0]
+        elif len(vals) < 1:
+            raise CatalogueEntityNotFoundException("No such package")
+        else:
+            return vals
 
     @staticmethod
     def _validate_fields(fields):
