@@ -1,3 +1,7 @@
+#
+# Copyright (C) 2020 IHS Markit.
+# All Rights Reserved
+#
 import logging
 import os
 
@@ -10,11 +14,9 @@ import multiprocessing
 from concurrent.futures import ThreadPoolExecutor
 from wrapt import ObjectProxy
 from datetime import datetime, timedelta
-from dateutil.tz import tzlocal
+from dateutil import tz
 from boto3 import Session
-from botocore.exceptions import ClientError
-from botocore.session import get_session
-from botocore.credentials import RefreshableCredentials
+import botocore
 from urllib.parse import urlparse, ParseResult
 from dli.client.exceptions import InsufficientPrivilegesException, S3FileDoesNotExist
 
@@ -35,7 +37,7 @@ class Client:
                 # expiring the token after an hour. It's conservative
                 # but it should just be refreshed. TODO add expiry time
                 # to the get_s3_access_keys_for_dataset endpoint
-                datetime.now().replace(tzinfo=tzlocal()) + timedelta(hours=1)
+                datetime.now().replace(tzinfo=tz.tzlocal()) + timedelta(hours=1)
             ).isoformat()
 
             return dict(
@@ -45,13 +47,13 @@ class Client:
                 expiry_time=expiry_time
             )
 
-        session_credentials = RefreshableCredentials.create_from_metadata(
+        session_credentials = botocore.credentials.RefreshableCredentials.create_from_metadata(
             metadata=_credentials(),
             refresh_using=_credentials,
             method='NOOP'
         )
 
-        session = get_session()
+        session = botocore.session.get_session()
         # Extremely dirty but it seems to be the 'blessed' way of doing this.
         session._credentials = session_credentials
         return cls(Session(botocore_session=session))
@@ -67,11 +69,36 @@ class Client:
         self.s3fs = s3fs.S3FileSystem()
         self.s3fs.s3 = self.s3_client
 
-    def download_files_from_s3_path(self, s3_path, destination):
+    def download_files_from_s3_path(self, s3_path, destination, flatten=False):
         """
         Helper function to download a file as a stream from S3
-        :param s3_path, required, example: s3://bucket_name/prefix_part1/prefix_part2/file_name.txt
-        :param destination, required, must be a directory, if doesn't exist, will be created
+        :param str s3_path: required, example: s3://bucket_name/prefix_part1/prefix_part2/file_name.txt
+
+        :param str destination: required. The path on the system, where the
+            files should be saved. must be a directory, if doesn't exist, will
+            be created.
+
+        :param bool flatten: The default behaviour (=False) is to use the s3
+            file structure when writing the downloaded files to disk. Example:
+            [
+              'storm/climate/storm_data/storm_fatalities/as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
+              'storm/climate/storm_data/storm_fatalities/as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
+            ]
+
+            When flatten = True, we remove the s3 structure. Example:
+
+            Example output for new behaviour:
+            [
+              './storm-flattened/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
+              './storm-flattened/StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
+            ]
+
+        :returns: a list of the paths where the files were downloaded
+            to e.g. ['path/git /package/dataset/as_of_date=2019-10-16
+            /file.csv.gz'].
+            with flatten=True e.g. ['path/file.csv.gz']
+
+        :rtype: str
         """
         if os.path.isfile(destination):
             raise NotADirectoryError(destination)
@@ -79,22 +106,37 @@ class Client:
         parse_result = urlparse(s3_path)
         bucket = parse_result.netloc
         # Removes forward slash
-        path = parse_result.path.lstrip('/').rstrip('/')
+        download_path = parse_result.path.lstrip('/').rstrip('/')
 
         def _download(obj_summary):
             # Ignore 'folders'
             key = obj_summary['Key']
+            destination_key = key
             if not key.endswith('/'):
-                destinatation_path = os.path.join(destination, key)
+
+                def flatten_s3_file_path(s3_path):
+                    from urllib.parse import urlparse
+                    import ntpath
+                    path = urlparse(s3_path)[2]  # [scheme, netloc, path, ...]
+                    head, tail = ntpath.split(path)
+                    file = tail or ntpath.basename(head)
+                    return file
+
+                if flatten:
+                    destination_key = flatten_s3_file_path(s3_path=key)
+
+                destinatation_path = os.path.join(destination, destination_key)
                 subfolder = os.path.dirname(destinatation_path)
                 os.makedirs(subfolder, exist_ok=True)
                 logger.info('Downloading File %s to %s', key, destinatation_path)
                 self.s3_client.download_file(bucket, key, destinatation_path)
+                return destinatation_path
 
-        keys = list(self._list_objects(bucket, path))
+        keys = list(self._list_objects(bucket, download_path))
         threads = min(THREADS, len(keys))
         with ThreadPoolExecutor(max_workers=threads) as executor:
-            list(executor.map(_download, keys))
+            destination_path = list(executor.map(_download, keys))
+        return destination_path
 
     def upload_files_to_s3(self, files, s3_location, token_refresher=None, num_retries=None):
         """
@@ -140,7 +182,7 @@ class Client:
                 self.s3_client.upload_file(
                     file_path, bucket.name, target.lstrip('/')
                 )
-            except ClientError as e:
+            except botocore.exceptions.ClientError as e:
                 if e.response['Error']['Code'] == 'AccessDenied':
                     raise InsufficientPrivilegesException() from e
 
@@ -177,14 +219,18 @@ class Client:
 
             if os.path.isfile(file):
                 logger.info("detected file: %s", file)
-                files_to_upload.append({'file': file, 's3_suffix': os.path.basename(file)})
+                files_to_upload.append({'file': file,
+                                        's3_suffix': os.path.basename(file)})
             elif os.path.isdir(file):
                 logger.info("detected directory: %s", file)
                 all_contents = glob.glob(os.path.join(file, "**/*"), recursive=True)
                 fs = [f for f in all_contents if os.path.isfile(f)]
 
                 for f in fs:
-                    files_to_upload.append({'file': f, 's3_suffix': os.path.relpath(f, file)})
+                    files_to_upload.append({
+                        'file': f,
+                        's3_suffix': os.path.relpath(f, file).replace("\\", "/")
+                    })
 
         return files_to_upload
 
