@@ -1,17 +1,27 @@
-import logging
+#
+# Copyright (C) 2020 IHS Markit.
+# All Rights Reserved
+#
 import os
 import warnings
+import requests
+import logging
 
 from dli.client import _setup_logging
 from dli.client.dli_client import DliClient
 from dli import __version__
-import requests
 
+
+logger = logging.getLogger(__name__)
 DEPRECATED_AGE = 1
 
 
+def _set_pypi_url(package_name):
+    return f"https://pypi.python.org/pypi/{package_name}/json"
+
+
 def version_check(package_name):
-    url = f"https://pypi.python.org/pypi/{package_name}/json"
+    url = _set_pypi_url(package_name)
 
     try:
         data = requests.get(url).json()
@@ -22,35 +32,67 @@ def version_check(package_name):
         offset = versions.index(__version__)
 
         if offset > DEPRECATED_AGE:
-            warnings.warn("You are using an old version of the SDK, please upgrade"
-                          "using `pip install [dli] --upgrade` "
+            warnings.warn("You are using an old version of the SDK, please upgrade "
+                          "using `pip install dli --upgrade` "
                           "before the SDK no longer functions as expected",
                           PendingDeprecationWarning)
-    except:
+
+    except requests.exceptions.SSLError as e:
         # Version check has been see to fail by Gilberto with an SSLError
         # when doing the handshake with pypi.
+        logger.warning(f'SSLError pypi version_check:',
+                       extra={'SSLError': e})
+    except Exception:
         pass
 
 
-class PEBCAKWarning(UserWarning):
-    pass
-
-
-def start_session(
+def _start_session(
     api_key=None,
     root_url="https://catalogue.datalake.ihsmarkit.com/__api",
     host=None,
     debug=False,
     strict=True,
+    use_keyring=True,
+    log_level=None,
 ):
+    if log_level is None:
+        log_level = ':error'
+    logger = _setup_logging(log_level)
+
+    version_check('dli')
+
+    user = os.environ.get("DLI_ACCESS_KEY_ID")
+    pasw = os.environ.get("DLI_SECRET_ACCESS_KEY")
+    api_key = api_key or os.environ.get("DLI_API_KEY")
+
+    logger.debug("Checking auth flow")
+
+    if (user is not None and pasw is not None) and api_key is None:
+        logger.debug("Using SAM client credentials authentication flow")
+        return get_client()(root_url, host, debug=debug, strict=strict,
+                            access_id=user, secret_key=pasw, use_keyring=use_keyring)
+    elif (user is None or pasw is None) and api_key is None:
+        logger.debug("Using SAM PKCE web authentication flow")
+        return get_client()(root_url, host, debug=debug, strict=strict, use_keyring=use_keyring)
+    elif api_key is not None:
+        logger.debug("Using API Key authentication flow")
+        if api_key is not None:
+            return get_client()(root_url, host, debug=debug, strict=strict,
+                                secret_key=api_key, use_keyring=use_keyring)
+
+
+def start_session(*args, **kwargs):
     """
+    .. deprecated:: 1.8.3
+
+
     Entry point for the Data Lake SDK, returns a client instance that
     can be used to consume or register datasets.
 
-    Example for starting a session:
+    Example for creating a client:
 
         from dli.client import session
-        dl = session.start_session(api_key)
+        client = session.start_session()
 
     :param str api_key: Your API key, can be retrieved from your dashboard in
                         the Catalogue UI.
@@ -68,38 +110,29 @@ def start_session(
     :param bool strict: Optional. When True, all exception messages and stack
                         trace are printed. When False, a shorter message is
                         printed and `None` should be returned.
+    :param bool use_keyring: Optional. When True, cache the JWT in the system keyring
+                             and retrieve that JWT from the system keyring if set.
+                             Otherwise disable the keyring completely.
 
     :returns: Data Lake interface client
     :rtype: dli.client.dli_client.DliClient
 
+    .. deprecated:: 1.8.3
+
+        `api_key`: please use the basic flow or client credentials.
+
     """
-    logger = _setup_logging('stdout:debug')
-    version_check('dli')
+    __doc__ = _start_session.__doc__
 
-    user = os.environ.get("DLI_ACCESS_KEY_ID")
-    pasw = os.environ.get("DLI_SECRET_ACCESS_KEY")
-    api_key = api_key or os.environ.get("DLI_API_KEY")
+    warnings.warn(
+        'start_session is deprecated. To create a client: \n\n'
+        '\t\timport dli\n'
+        '\t\tdli.connect()\n',
+        DeprecationWarning
+    )
 
-    logger.debug("Checking auth flow")
+    return _start_session(*args, **kwargs)
 
-    if (user is not None and pasw is not None) and api_key is None:
-        logger.debug("Using new application authentication flow")
-        return get_client()(root_url, host, debug=debug, strict=strict,
-                            access_id=user, secret_key=pasw)
-    elif (user is None or pasw is None) and api_key is None:
-        logger.debug("Using SAM application authentication flow")
-        url = "test"
-        print(f"If your browser has not been automatically opened, "
-              f"then please copy and paste this URL into your browser "
-              f"to complete the login process within the next 15 minutes:\n"
-              f"{url}")
-
-        return
-    elif api_key is not None:
-        logger.debug("Using old application authentication flow")
-        if api_key is not None:
-            return get_client()(root_url, host, debug=debug, strict=strict,
-                                secret_key=api_key)
 
 def get_client():
     return DliClient
