@@ -1,29 +1,48 @@
+#
+# Copyright (C) 2020 IHS Markit.
+# All Rights Reserved
+#
 import json
 import os
 import textwrap
 import uuid
 import warnings
-from typing import List
-from urllib.parse import urljoin
 import pandas
-from tabulate import tabulate
 import pyarrow
+import boto3
 
+from typing import List, Optional
+from urllib.parse import urljoin
+from tabulate import tabulate
+from s3transfer.manager import TransferManager
+
+
+from dli.aws import create_refreshing_session
+from dli.client import utils
 from dli.models import log_public_functions_calls_using, SampleData, \
     AttributesDict
 from dli.client.aspects import analytics_decorator, logging_decorator
 from dli.client.components.urls import consumption_urls, dataset_urls
-from dli.client.exceptions import DataframeStreamingException
+from dli.client.exceptions import DataframeStreamingException, \
+    UnAuthorisedAccessException
 from dli.models.dictionary_model import DictionaryModel
+from dli.models.file_model import get_or_create_os_path
+
+
+try:
+    import dask.dataframe as dask_dataframe_  # noqa
+except ImportError:
+    dask_dataframe_ = None
+
+import logging
+trace_logger = logging.getLogger('trace_logger')
+
 
 
-@log_public_functions_calls_using(
-    [analytics_decorator, logging_decorator], class_fields_to_log=['dataset_id']
-)
 class DatasetModel(AttributesDict):
 
     @property
-    def sample_data(self):
+    def sample_data(self) -> SampleData:
         return SampleData(self)
 
     @property
@@ -50,7 +69,7 @@ class DatasetModel(AttributesDict):
     @classmethod
     def _from_v1_response_to_v2(cls, v1_response):
         response = cls._client.session.get(
-            dataset_urls.v2_instance.format(
+            dataset_urls.v2_by_id.format(
                 id=v1_response['properties']['datasetId']
             )
         )
@@ -81,7 +100,51 @@ class DatasetModel(AttributesDict):
             dataset_id=dataset_id
         )
 
-    def download(self, destination_path, flatten=False):
+    def dask_dataframe(self):
+        """
+        Creates a dask dataframe from the datasets.
+        """
+
+        if not dask_dataframe_:
+            raise RuntimeError(
+                'Please install dask using `pip install dask'
+                ' or a preconfigured package with:\n\n\t pip install dli[dask]'
+            )
+
+        if self.data_format.lower() == 'parquet':
+            return dask_dataframe_.read_parquet(
+                f's3://{self.organisation_short_code}/{self.short_code}' + (
+                    '' if self.short_code.endswith('/') else '/'
+                ),
+                storage_options={
+                    'client_kwargs': {
+                        'endpoint_url': f'https://{self._client._environment.s3_proxy}',
+                    },
+                    'key': self._client.session.auth_key,
+                    'secret': 'noop',
+                }
+            )
+        if self.data_format.lower() == 'csv':
+            return dask_dataframe_.read_csv(
+                f's3://{self.organisation_short_code}/{self.short_code}' + (
+                    '' if self.short_code.endswith('/') else '/'
+                ),
+                storage_options={
+                    'client_kwargs': {
+                        'endpoint_url': f'https://{self._client._environment.s3_proxy}',
+                    },
+                    'key': self._client.session.auth_key,
+                    'secret': 'noop',
+                }
+            )
+
+
+    def download(
+        self, 
+        destination_path: str,
+        flatten: Optional[bool] = False,
+        filter_path: Optional[str] = None
+    ) -> List[str]:
         """
         Downloads all files from the latest instance into the provided
         destination path.
@@ -93,23 +156,129 @@ class DatasetModel(AttributesDict):
             be created.
 
         :param bool flatten: The default behaviour (=False) is to use the s3
-            file structure when writing the downloaded files to disk. Example:
-            [
-              'storm/climate/storm_data/storm_fatalities/as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
-              'storm/climate/storm_data/storm_fatalities/as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
-            ]
+            file structure when writing the downloaded files to disk. 
+
+        :param str filter_path: if provided only a subpath matching the filter_path
+            will be downloaded
+
+
+        :return: the list of the downloaded files
+
+
+        :example:
+
+            Downloading without flatten:
+
+            .. code-block:: python
+
+                >>> dataset.download('./local/path/')
+                [
+                  'as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
+                  'as_of_date=2019-09-11/type=full/StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
+                ]
+
+        :example:
+
+            Downloading with ``filter_path``
+
+            .. code-block:: python
+
+                >>> dataset.download(
+                    './local/path/', filter_path='as_of_date=2019-09-10/'
+                )
+                [
+                  'as_of_date=2019-09-10/type=full/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
+                ]
+
+
+        :example:
 
             When flatten = True, we remove the s3 structure. Example:
 
             Example output for new behaviour:
-            [
-              './storm-flattened/StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
-              './storm-flattened/StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
-            ]
 
-        :return: the list of the downloaded files
+            .. code-block:: python
+
+                >>> dataset.download('./local/path/', flatten=True)
+                [
+                  'StormEvents_details-ftp_v1.0_d1950_c20170120.csv.gz',
+                  'StormEvents_details-ftp_v1.0_d1951_c20160223.csv.gz'
+                ]
+
+
         """
-        return self.instances.latest().download(destination_path, flatten)
+
+        if not self.has_access:
+            raise Exception(
+                'Unfortunately the user you are using '
+                'does not have access to this dataset. '
+                'Please request access to the package/dataset '
+                "to be able to retrieve this content."
+            )
+
+
+        def add_request_id_to_session(**kwargs):
+            request_id = str(uuid.uuid4())
+            kwargs["request"].headers['X-Request-ID'] = request_id
+            trace_logger.info(
+                f'GET Request to https://{self._client._environment.s3_proxy} '
+                f'with request_id: {request_id}'
+            )
+
+        s3_resource = create_refreshing_session(
+            dli_client_session=self._client.session,
+            event_hooks=add_request_id_to_session
+
+        ).resource(
+            's3',
+            endpoint_url=f'https://{self._client._environment.s3_proxy}'
+        )
+
+        s3_client = s3_resource.meta.client
+        with TransferManager(s3_client) as transfer_manager:
+
+            bucket = s3_resource.Bucket(
+                self.organisation_short_code
+            )
+
+            _paths = []
+
+            filter_prefix = self.short_code + (
+                    '' if self.short_code.endswith('/') else '/'
+            )
+
+            if filter_path:
+                filter_prefix = filter_prefix + filter_path.lstrip('/')
+
+            # TODO parallelize this again?
+            for object_summary in bucket.objects.filter(
+                # Prefix searches for exact matches and folders
+                Prefix=filter_prefix
+            ):
+                if not object_summary.key.endswith('/'):
+                    to_path = get_or_create_os_path(
+                        object_summary.key,
+                        to=destination_path,
+                        flatten=flatten
+                    )
+
+                    print(f'Downloading {object_summary.key} to: {to_path}...')
+
+                    if os.path.exists(to_path):
+                        warnings.warn(
+                            'File already exists. Overwriting.'
+                        )
+
+                    # returns a future
+                    transfer_manager.download(
+                        self.organisation_short_code,
+                        object_summary.key,
+                        to_path
+                    )
+
+                    _paths.append(to_path)
+
+            return _paths
 
     def _dataframe(self, nrows=None, partitions: List[str] = None, raise_=True):
         warnings.warn(
@@ -119,7 +288,7 @@ class DatasetModel(AttributesDict):
         )
         self.dataframe(nrows=nrows, partitions=partitions, raise_=raise_)
 
-    def dataframe(self, nrows=None, partitions: List[str] = None, raise_=True):
+    def dataframe(self, nrows=None, partitions: List[str] = None, raise_=True) -> 'pandas.DataFrame':
         """
         Return the data from the files in the latest instance of the dataset
         as a pandas DataFrame.
@@ -138,15 +307,42 @@ class DatasetModel(AttributesDict):
             default None.
 
         :param List[str] partitions: Optional. A dict of filters (partitions) to
-            apply to the dataframe request in the form of: ["a=12","b>20190110"]
-            - will permit whitespace and equality operators [<=, <, =, >, >=]
+            apply to the dataframe request in the form of: `["a=12","b>20190110"]`
+            - will permit whitespace and equality operators `[<=, <, =, >, >=]`
 
         :param bool raise_: Optional. Raise exception if the dataframe stream
             stopped prematurely
 
-        :returns: Pandas DataFrame.
-        :rtype: pandas.DataFrame
+        :example:
+
+            Basic usage:
+
+            .. code-block:: python
+
+                    dataframe = dataset.dataframe()
+
+        :example:
+
+            Dataframe filtered by partition with nrows (partitions can be fetched
+            via `dataset.partitions()`:
+
+            .. code-block:: python
+
+                    dataframe = dataset.dataframe(
+                        nrows=1000,
+                        partitions=["as_of_date=2017-03-07"]
+                    )
+
         """
+
+        if not self.has_access:
+            raise Exception(
+                'Unfortunately the user you are using '
+                'does not have access to this dataset. '
+                'Please request access to the package/dataset '
+                "to be able to retrieve this content."
+            )
+
         params = {}
 
         if nrows is not None:
@@ -160,20 +356,9 @@ class DatasetModel(AttributesDict):
             consumption_urls.consumption_dataframe.format(id=self.id)
         )
 
-        # Generate request_id on the SDK side for use in Consumption log
-        # messages. This is useful for debugging what seems like a success
-        # (no exception) but is not what the user wanted e.g. not the data
-        # they expected.
-        request_id = str(uuid.uuid1())
-
-        tracing = os.environ.get('DLI_TRACE', '')
-        if tracing and tracing.lower() == 'true':
-            print(f"DEBUG: Your REQUEST ID is {request_id}")
-
         response = self._client.session.get(
             dataframe_url, stream=True,
             params=params,
-            headers={'X-Request-ID': request_id},
         )
 
         # Don't decode content. We would like to keep the raw
@@ -226,7 +411,7 @@ class DatasetModel(AttributesDict):
         )
         self.partitions()
 
-    def partitions(self):
+    def partitions(self) -> dict:
         """
         Retrieves the list of available partitions for a given dataset.
 
@@ -238,11 +423,15 @@ class DatasetModel(AttributesDict):
         the options `us`, `eu` and `asia` can be separated into S3 paths like
         so:
 
+        .. code-block::
+
             package-name/dataset/as_of_date=2019-09-10/location=eu/filename.csv
             package-name/dataset/as_of_date=2019-09-10/location=us/filename.csv
 
         in this case the `partitions` will be returned as:
 
+        .. code-block::
+
             {'as_of_date': ['2019-09-10'], 'location': ['eu', 'us]}
         """
         response = self._client.session.get(
@@ -309,6 +498,9 @@ class DatasetModel(AttributesDict):
         else:
             print("No columns/info available.")
 
+    def metadata(self):
+        utils.print_model_metadata(self)
+
     def __repr__(self):
         return f'<Dataset short_code={self.short_code}>'
 
@@ -325,3 +517,9 @@ class DatasetModel(AttributesDict):
                f"\n" \
                f"{splits}\n" \
                f"{separator}"
+
+
+log_public_functions_calls_using(
+    [analytics_decorator, logging_decorator],
+    class_fields_to_log=['dataset_id']
+)(DatasetModel)
