@@ -30,6 +30,7 @@ def jwt(request):
     aud = os.environ[f'{request.param}_OIDC_AUD']
     return {
         "datalake": {
+            "user_id": "3e1c688b-86f3-4b60-b988-15cb2d9b0133",
             "accounts": {
                 "datalake-test1": "rw",
                 "datalake-test2": "rw",
@@ -59,11 +60,12 @@ class TestDliClient(DliClient):
 
     def _new_session(self):
         return Session(
-            None,
-            self.secret_key,
-            self._environment,
-            self.host,
-            self.auth_key,
+            access_id=None,
+            secret_key=self.secret_key,
+            environment=self._environment,
+            host=self.host,
+            auth_key=self.auth_key,
+            logger=self.logger,
         )
 
     @contextmanager
@@ -82,8 +84,6 @@ class TestDliClient(DliClient):
         )
 
 
-
-
 @pytest.fixture
 def client(monkeypatch, jwt, request):
     # This is the S3 address used for the QA environment.
@@ -134,13 +134,13 @@ def package(client):
         manager_id='datalake-test2'
     )
     yield package
-    client.delete_package(package.package_id)
+    client.delete_package(package.id)
 
 
 @pytest.fixture
 def csv_dataset_builder(package):
     dataset_builder = DatasetBuilder(
-        package_id=package.package_id,
+        package_id=package.id,
         name='dataset-files-test-' + random_name(),
         description="My dataset description",
         content_type="Pricing",
@@ -164,7 +164,19 @@ def empty_dataset(client, aws_roles, csv_dataset_builder):
 
 
 @pytest.fixture
-def csv_dataset(client, aws_roles, csv_dataset_builder):
+def csv_dataset(client, csv_dataset_builder):
+    aws_account = {
+        'awsAccountId': aws_envs.PROD,
+        'awsAccountName': 'dl_prod_aws_account',
+        'awsRoleArn': 'arn:aws:iam::867407613858:role/trust-entity-dev-test',
+        'awsRegion': 'eu-west-1',
+        'accountIds': ['datalake-test1', 'datalake-test2', 'datalake-mgmt']
+    }
+
+    client.session.post(
+        '__api/me/aws-accounts', json=aws_account
+    )
+
     dataset = client.register_dataset(csv_dataset_builder)
     # NOTE register_s3_datafile both registers the datafile in the
     # datalake and it uploads the file to the bucket.
